@@ -284,16 +284,21 @@ pub fn bind_socket_to_interface(socket: &Socket, info: &InterfaceInfo, ipv6: boo
         })
     }
 
-    #[cfg(target_os = "freebsd")]
+    #[cfg(any(target_os = "freebsd", target_os = "netbsd"))]
     {
         let _ = (socket, info, ipv6); // Suppress unused warnings
         Err(anyhow!(
-            "Interface binding (-i) is not supported on FreeBSD. \
-             FreeBSD lacks SO_BINDTODEVICE and IP_BOUND_IF socket options."
+            "Interface binding (-i) is not supported on FreeBSD/NetBSD. \
+             These platforms lack SO_BINDTODEVICE and IP_BOUND_IF socket options."
         ))
     }
 
-    #[cfg(not(any(target_os = "linux", target_os = "macos", target_os = "freebsd")))]
+    #[cfg(not(any(
+        target_os = "linux",
+        target_os = "macos",
+        target_os = "freebsd",
+        target_os = "netbsd"
+    )))]
     {
         let _ = (socket, info, ipv6); // Suppress unused warnings
         Err(anyhow!(
